--==============================================================================
--
--     File Name: mptl_if_target_netlist_wrap_admxrc7k1x_master.vhd
--       Version: 0.1
--          Date: 21 Mar 2012
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Target MPTL interface netlist wrapper
--
--   Description: Wrapper for target MPTL interface simulation netlist
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 21 Mar 2012: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity mptl128_interface_target_7v1x_master is
port(
  -- OCP Interface
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
  direct_master_m2s        : in    adb3_ocp_m2sT;
  direct_master_s2m        : out   adb3_ocp_s2mT;
  -- MPTL Interface
  mptl_tx                  : out   mptl_pins_t;
  mptl_rx                  : in    mptl_pins_t;
  mptl_clk                 : in    std_logic_dbl_t;
  mptl_clk_out             : out   std_logic;
  -- Bridge to Target Sideband Signals
  mptl_target_configured_n : out   std_logic;
  mptl_target_gtp_online_n : out   std_logic;
  mptl_bridge_gtp_online_n : in    std_logic;
  -- MPTL64 Sideband Interface
  gpio_t2b                 : in    std_logic_vector(15 downto 0);
  gpio_b2t                 : out   std_logic_vector(15 downto 0));
end;

architecture sim_admxrc7v1x of mptl128_interface_target_7v1x_master is

  signal direct_slave_m2s_slv : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
  signal direct_slave_s2m_slv : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
  signal dma_channels_m2s_slv : std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0);
  signal dma_channels_s2m_slv : std_logic_vector(DMA_CHANNELS*adb3_ocp_s2mT_slv_length-1 downto 0);
  signal direct_master_m2s_slv : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
  signal direct_master_s2m_slv : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);

begin

  -- Convert OCP channels between record and SLV structure
  -- Define response data from target FPGA during full MPTL simulation
  direct_slave_m2s     <= to_adb3_ocp_m2sT(direct_slave_m2s_slv);
  direct_slave_s2m_slv <= to_std_logic_vector(make_defined_s2m(direct_slave_s2m));

  dma_channels_m2s     <= to_adb3_ocp_m2sT_array(dma_channels_m2s_slv);
  dma_channels_s2m_slv <= to_std_logic_vector(make_defined_s2m_array(dma_channels_s2m));

  direct_master_m2s_slv <= to_std_logic_vector(make_defined_m2s(direct_master_m2s));
  direct_master_s2m     <= to_adb3_ocp_s2mT(direct_master_s2m_slv);

  mptl128_interface_target_7v1x_master_slv_i: entity work.mptl128_interface_target_7v1x_master_slv
  port map (
    ocp_clk                  => ocp_clk,
    direct_slave_m2s         => direct_slave_m2s_slv,
    direct_slave_s2m         => direct_slave_s2m_slv,
    dma_channels_m2s         => dma_channels_m2s_slv,
    dma_channels_s2m         => dma_channels_s2m_slv,
    dma_abort                => dma_abort,
    direct_master_m2s        => direct_master_m2s_slv,
    direct_master_s2m        => direct_master_s2m_slv,
    mptl_tx_p                => mptl_tx.p(MPTL_SER_WIDTH-1 downto 0),
    mptl_tx_n                => mptl_tx.n(MPTL_SER_WIDTH-1 downto 0),
    mptl_rx_p                => mptl_rx.p(MPTL_SER_WIDTH-1 downto 0),
    mptl_rx_n                => mptl_rx.n(MPTL_SER_WIDTH-1 downto 0),
    mptl_clk_p               => mptl_clk.p,
    mptl_clk_n               => mptl_clk.n,
    mptl_clk_out             => mptl_clk_out,
    mptl_target_configured_n => mptl_target_configured_n,
    mptl_target_gtp_online_n => mptl_target_gtp_online_n,
    mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n,
    gpio_t2b                 => gpio_t2b,
    gpio_b2t                 => gpio_b2t);

end;