--==============================================================================
--
--     File Name: mptl_if_target_wrap_admxrc7v1x.vhd
--       Version: 0.2
--          Date: 09 Nov 2012
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Target MPTL interface wrapper
--
--   Description: Instantiate full-MPTL simulation netlist or synthesis
--                core for target MPTL interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 26 Jun 2012: Initial design
--                0.2, TW, 09 Nov 2012:
--                  - Added support for _slv version of MPTL core.
--                  - MPTL core instances are now named
--                    mptl128_interface_target_i in order to permit the same
--                    constraints to work for H and X variants of MPTL core.
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity mptl_if_target_wrap is
generic(
  enable_dm : boolean := false);
port(
  -- OCP Interface
  ocp_clk           : in    std_logic;
  direct_slave_m2s  : out   adb3_ocp_m2sT;
  direct_slave_s2m  : in    adb3_ocp_s2mT;
  dma_channels_m2s  : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m  : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  dma_abort         : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
  direct_master_m2s : in    adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  direct_master_s2m : out   adb3_ocp_s2mT;
  -- MPTL Interface
  mptl_b2t          : in    mptl_b2t_t;
  mptl_t2b          : out   mptl_t2b_t;
  mptl_clk          : in    std_logic_dbl_t;
  -- MPTL Initialisation
  ocp_ready         : in    std_logic;
  mptl_ready        : out   std_logic;
  mptl_sb_b2t       : in    mptl_sb_b2t_t;
  mptl_sb_t2b       : out   mptl_sb_t2b_t;
  -- MPTL Sideband
  gpio_t2b          : in    std_logic_vector(15 downto 0);
  gpio_b2t          : out   std_logic_vector(15 downto 0));
end;


architecture syn_admxrc7v1x of mptl_if_target_wrap is

  constant USE_SLV : boolean := true;

  -- Unused IO
  signal mptl_clk_out : std_logic;

  -- MPTL sideband
  signal mptl_target_config_n     : std_logic;
  signal mptl_target_gtp_online_n : std_logic;
  signal mptl_bridge_gtp_online_n : std_logic;

  -- MPTL ready logic
  signal mptl_bridge_gtp_online_sr_n : std_logic_vector(31 downto 0) := (others => '1');
  signal mptl_target_gtp_online_sr_n : std_logic_vector(31 downto 0) := (others => '1');
  signal i_mptl_ready : std_logic := '0';

  component mptl128_interface_target_7v1x
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_7v1x_slv
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
    direct_slave_s2m         : in    std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
    dma_channels_m2s         : out   std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0);
    dma_channels_s2m         : in    std_logic_vector(DMA_CHANNELS*adb3_ocp_s2mT_slv_length-1 downto 0);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_tx_p                : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_tx_n                : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_rx_p                : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_rx_n                : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_clk_p               : in    std_logic;
    mptl_clk_n               : in    std_logic;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_7v1x_master
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    direct_master_m2s        : in    adb3_ocp_m2sT;
    direct_master_s2m        : out   adb3_ocp_s2mT;
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_7v1x_master_slv
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
    direct_slave_s2m         : in    std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
    dma_channels_m2s         : out   std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0);
    dma_channels_s2m         : in    std_logic_vector(DMA_CHANNELS*adb3_ocp_s2mT_slv_length-1 downto 0);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    direct_master_m2s        : in    std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
    direct_master_s2m        : out   std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
    -- MPTL Interface
    mptl_tx_p                : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_tx_n                : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_rx_p                : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_rx_n                : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_clk_p               : in    std_logic;
    mptl_clk_n               : in    std_logic;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

begin

  -- Hold off MPTL target configured until all OCP clocks are stable (resets inactive)
  mptl_sb_t2b.mptl_target_configured_l <= mptl_target_config_n or not ocp_ready;

  -- Connect MPTL initialisation signals
  mptl_sb_t2b.mptl_target_gtp_online_l <= mptl_target_gtp_online_n;
  mptl_bridge_gtp_online_n <= mptl_sb_b2t.mptl_bridge_gtp_online_l;

  -- 'mptl_ready' output can be used to reset application logic, but must ignore short retry pulses
  gen_mptl_ready : process (mptl_clk_out)
  begin  -- process
    if rising_edge(mptl_clk_out) then
       mptl_bridge_gtp_online_sr_n <= mptl_bridge_gtp_online_sr_n(30 downto 0) & mptl_bridge_gtp_online_n;
       mptl_target_gtp_online_sr_n <= mptl_target_gtp_online_sr_n(30 downto 0) & mptl_target_gtp_online_n;
       i_mptl_ready <= NOT (mptl_bridge_gtp_online_sr_n(0) AND mptl_bridge_gtp_online_sr_n(31)) AND
                       NOT (mptl_target_gtp_online_sr_n(0) AND mptl_target_gtp_online_sr_n(31));
    end if;
  end process;

  mptl_ready <= i_mptl_ready;

  use_slv_true : if USE_SLV generate
    signal ds_m2s_slv : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
    signal ds_s2m_slv : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
    signal dma_m2s_slv : std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0);
    signal dma_s2m_slv : std_logic_vector(DMA_CHANNELS*adb3_ocp_s2mT_slv_length-1 downto 0);
  begin
    dm_enable_false : if NOT enable_dm generate
      mptl128_interface_target_i : mptl128_interface_target_7v1x_slv
      port map(
        -- OCP Interface
        ocp_clk                  => ocp_clk,                  -- in    std_logic
        direct_slave_m2s         => ds_m2s_slv,               -- out   std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0)
        direct_slave_s2m         => ds_s2m_slv,               -- in    std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0)
        dma_channels_m2s         => dma_m2s_slv,              -- out   std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0)
        dma_channels_s2m         => dma_s2m_slv,              -- in    std_logic_vector(DMA_CHANNELS*adb3_ocp_s2mT_slv_length-1 downto 0)
        dma_abort                => dma_abort,                -- in    std_logic_vector(DMA_CHANNELS-1 downto 0)
        -- MPTL Interface
        mptl_tx_p                => mptl_t2b.p,               -- out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_tx_n                => mptl_t2b.n,               -- out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_rx_p                => mptl_b2t.p,               -- in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_rx_n                => mptl_b2t.n,               -- in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_clk_p               => mptl_clk.p,               -- in    std_logic
        mptl_clk_n               => mptl_clk.n,               -- in    std_logic
        mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
        -- MPTL Sideband Signals
        mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
        mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
        -- MPTL Sideband Interface
        gpio_t2b                 => gpio_t2b,                 -- in    std_logic_vector(15 downto 0)
        gpio_b2t                 => gpio_b2t);                -- out   std_logic_vector(15 downto 0)

        direct_master_s2m <= ADB3_OCP_DEFAULT_S2M;
    end generate;

    dm_enable_true : if enable_dm generate
      signal dm_m2s_slv : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
      signal dm_s2m_slv : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
    begin
      mptl128_interface_target_i : mptl128_interface_target_7v1x_master_slv
      port map(
        -- OCP Interface
        ocp_clk                  => ocp_clk,                  -- in    std_logic
        direct_slave_m2s         => ds_m2s_slv,               -- out   std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0)
        direct_slave_s2m         => ds_s2m_slv,               -- in    std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0)
        dma_channels_m2s         => dma_m2s_slv,              -- out   std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0)
        dma_channels_s2m         => dma_s2m_slv,              -- in    std_logic_vector(DMA_CHANNELS*adb3_ocp_m2sT_slv_length-1 downto 0)
        dma_abort                => dma_abort,                -- in    std_logic_vector(DMA_CHANNELS-1 downto 0)
        direct_master_m2s        => dm_m2s_slv,               -- in    std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0)
        direct_master_s2m        => dm_s2m_slv,               -- out   std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0)
        -- MPTL Interface
        mptl_tx_p                => mptl_t2b.p,               -- out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_tx_n                => mptl_t2b.n,               -- out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_rx_p                => mptl_b2t.p,               -- in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_rx_n                => mptl_b2t.n,               -- in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0)
        mptl_clk_p               => mptl_clk.p,               -- in    std_logic
        mptl_clk_n               => mptl_clk.n,               -- in    std_logic
        mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
        -- MPTL Sideband Signals
        mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
        mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
        -- MPTL Sideband Interface
        gpio_t2b                 => gpio_t2b,                 -- in    std_logic_vector(15 downto 0)
        gpio_b2t                 => gpio_b2t);                -- out   std_logic_vector(15 downto 0)

      direct_master_s2m <= to_adb3_ocp_s2mT(dm_s2m_slv);
      dm_m2s_slv <= to_std_logic_vector(make_defined_m2s(direct_master_m2s));
    end generate;

    direct_slave_m2s <= to_adb3_ocp_m2sT(ds_m2s_slv);
    ds_s2m_slv <= to_std_logic_vector(make_defined_s2m(direct_slave_s2m));

    dma_channels_m2s <= to_adb3_ocp_m2sT_array(dma_m2s_slv);
    dma_s2m_slv <= to_std_logic_vector(make_defined_s2m_array(dma_channels_s2m));
  end generate;

  use_slv_false : if NOT USE_SLV generate
    dm_enable_false : if NOT enable_dm generate
      mptl128_interface_target_i : mptl128_interface_target_7v1x
      port map(
        -- OCP Interface
        ocp_clk                  => ocp_clk,                  -- in    std_logic
        direct_slave_m2s         => direct_slave_m2s,         -- out   adb3_ocp_m2sT
        direct_slave_s2m         => direct_slave_s2m,         -- in    adb3_ocp_s2mT
        dma_channels_m2s         => dma_channels_m2s,         -- out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
        dma_channels_s2m         => dma_channels_s2m,         -- in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
        dma_abort                => dma_abort,                -- in    std_logic_vector(DMA_CHANNELS-1 downto 0)
        -- MPTL Interface
        mptl_tx                  => mptl_t2b,                 -- out   mptl_pins_t
        mptl_rx                  => mptl_b2t,                 -- in    mptl_pins_t
        mptl_clk                 => mptl_clk,                 -- in    std_logic_dbl_t
        mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
        -- MPTL Sideband Signals
        mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
        mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
        -- MPTL Sideband Interface
        gpio_t2b                 => gpio_t2b,                 -- in    std_logic_vector(15 downto 0)
        gpio_b2t                 => gpio_b2t);                -- out   std_logic_vector(15 downto 0)

        direct_master_s2m <= ADB3_OCP_DEFAULT_S2M;
    end generate;

    dm_enable_true : if enable_dm generate
      mptl128_interface_target_i : mptl128_interface_target_7v1x_master
      port map(
        -- OCP Interface
        ocp_clk                  => ocp_clk,                  -- in    std_logic
        direct_slave_m2s         => direct_slave_m2s,         -- out   adb3_ocp_m2sT
        direct_slave_s2m         => direct_slave_s2m,         -- in    adb3_ocp_s2mT
        dma_channels_m2s         => dma_channels_m2s,         -- out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
        dma_channels_s2m         => dma_channels_s2m,         -- in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
        dma_abort                => dma_abort,                -- in    std_logic_vector(DMA_CHANNELS-1 downto 0)
        direct_master_m2s        => direct_master_m2s,        -- in    adb3_ocp_m2sT
        direct_master_s2m        => direct_master_s2m,        -- out   adb3_ocp_s2mT
        -- MPTL Interface
        mptl_tx                  => mptl_t2b,                 -- out   mptl_pins_t
        mptl_rx                  => mptl_b2t,                 -- in    mptl_pins_t
        mptl_clk                 => mptl_clk,                 -- in    std_logic_dbl_t
        mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
        -- MPTL Sideband Signals
        mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
        mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
        -- MPTL Sideband Interface
        gpio_t2b                 => gpio_t2b,                 -- in    std_logic_vector(15 downto 0)
        gpio_b2t                 => gpio_b2t);                -- out   std_logic_vector(15 downto 0)
    end generate;
  end generate;

end;
