--==============================================================================
--
--     File Name: ddr3_if_bank_admxrc7k1_mig7_v1_8.vhd
--       Version: 0.3
--          Date: 14 Feb 2013
--         Model: Synthesisable
--
--  Dependencies: Xilinx Coregen MIG DDR3 SDRAM interface
--
--       Company: Alpha Data
--        Design: On-board DDR3 SDRAM interface block using Xiling MIG cores
--
--   Description: OCP to MIG to DDR3 SDRAM interface for ADM-XRC-7K1
--
--   Limitations: MIG7 v1.8 requires ISE14.4 or later
--
--         Notes: MIG 7 Series - Release Notes and Known Issues for all releases
--                http://www.xilinx.com/support/answers/45195.htm
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 14 Feb 2013: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_comp.all;
use work.adb3_target_inc_pkg.all;
use work.ddr3_if_pkg.all;

entity ddr3_if_bank is
generic(
  sim   : boolean := false;
  clock : mig_clocks_t := MIG_CLOCKS_1600;
  bank  : integer range 0 to DDR3_BANKS-1 := 0);
port(
  -- OCP interface
  ocp_rst               : in    std_logic;
  ocp_clk               : in    std_logic;
  ocp_m2s               : in    adb3_ocp_m2sT;
  ocp_s2m               : out   adb3_ocp_s2mT;
  -- Memory interface
  ddr3_rst              : in    std_logic;
  ddr3_ref_clk          : in    std_logic;
  ddr3_clk              : in    std_logic;
  ddr3_iodelay_ctrl_rdy : in    std_logic;
  ddr3_xadc_temp_in     : in    std_logic_vector(11 downto 0) := (others => '0');
  ddr3_xadc_temp_out    : out   std_logic_vector(11 downto 0);
  -- Memory status
  ddr3_if_rdy           : out   std_logic;
  ddr3_if_stat          : out   std_logic_vector(3 downto 0);
  ddr3_if_err           : out   std_logic_vector(3 downto 0);
  -- Physical memory interface
  ddr3_addr_out         : out   ddr3_addr_out_t;
  ddr3_ctrl_out         : out   ddr3_ctrl_out_t;
  ddr3_data_inout       : inout ddr3_data_inout_t;
  ddr3_clk_out          : out   ddr3_clk_out_t;
  -- Debug info
  ddr3_if_debug         : out   std_logic_vector(31 downto 0));
end;


architecture syn_admxrc7k1 of ddr3_if_bank is

  component mig_7series_v1_8_infrastructure
  generic(
    SIMULATION     : string;
    TCQ            : integer;
    nCK_PER_CLK    : integer;
    CLKIN_PERIOD   : integer;
    CLKFBOUT_MULT  : integer;
    DIVCLK_DIVIDE  : integer;
    CLKOUT0_PHASE  : real;
    CLKOUT0_DIVIDE : integer;
    CLKOUT1_DIVIDE : integer;
    CLKOUT2_DIVIDE : integer;
    CLKOUT3_DIVIDE : integer;
    RST_ACT_LOW    : integer);
  port(
    clk              : out    std_logic;
    mem_refclk       : out    std_logic;
    freq_refclk      : out    std_logic;
    sync_pulse       : out    std_logic;
    auxout_clk       : out    std_logic;
    pll_locked       : out    std_logic;
    rst_phaser_ref   : out    std_logic;
    rstdiv0          : out    std_logic;
    mmcm_clk         : in     std_logic;
    sys_rst          : in     std_logic;
    iodelay_ctrl_rdy : in     std_logic;
    ref_dll_lock     : in     std_logic);
  end component;

  component mig_7series_v1_8_tempmon
  generic(
    TCQ              : integer;
    TEMP_MON_CONTROL : string;
    XADC_CLK_PERIOD  : integer;
    tTEMPSAMPLE      : integer);
  port(
    clk           : in     std_logic;
    xadc_clk      : in     std_logic;
    rst           : in     std_logic;
    device_temp_i : in     std_logic_vector(11 downto 0);
    device_temp   : out    std_logic_vector(11 downto 0));
  end component;

  component mig_7series_v1_8_memc_ui_top_std
  generic(
    TCQ                 : integer;
    ADDR_CMD_MODE       : string;
--    AL                  : string;
    PAYLOAD_WIDTH       : integer;
    BANK_WIDTH          : integer;
    BM_CNT_WIDTH        : integer;
    BURST_MODE          : string;
--    BURST_TYPE          : string;
--    CA_MIRROR           : string;
    CK_WIDTH            : integer;
    COL_WIDTH           : integer;
--    CMD_PIPE_PLUS1      : string;
    CS_WIDTH            : integer;
    nCS_PER_RANK        : integer;
    CKE_WIDTH           : integer;
    DATA_WIDTH          : integer;
    DATA_BUF_ADDR_WIDTH : integer;
--    DDR2_DQSN_ENABLE    : string;
    DM_WIDTH            : integer;
    DQ_CNT_WIDTH        : integer;
    DQ_WIDTH            : integer;
    DQS_CNT_WIDTH       : integer;
    DQS_WIDTH           : integer;
--    DRAM_TYPE           : string;
--    DRAM_WIDTH          : integer;
--    ECC                 : string;
--    ECC_WIDTH           : integer;
--    ECC_TEST            : string;
    MC_ERR_ADDR_WIDTH   : integer;
    REFCLK_FREQ         : real;
--    nAL                 : integer;
--    nBANK_MACHS         : integer;
--    CKE_ODT_AUX         : string;
    nCK_PER_CLK         : integer;
    ORDERING            : string;
    OUTPUT_DRV          : string;
--    IBUF_LPWR_MODE      : string;
--    IODELAY_HP_MODE     : string;
    DATA_IO_IDLE_PWRDWN : string;
--    BANK_TYPE           : string;
    DATA_IO_PRIM_TYPE   : string;
    IODELAY_GRP         : string;
    REG_CTRL            : string;
    RTT_NOM             : string;
    RTT_WR              : string;
    CL                  : integer;
    CWL                 : integer;
    tCK                 : integer;
    tCKE                : integer;
    tFAW                : integer;
--    tPRDI               : integer;
    tRAS                : integer;
    tRCD                : integer;
    tREFI               : integer;
    tRFC                : integer;
    tRP                 : integer;
    tRRD                : integer;
--    tRTP                : integer;
--    tWTR                : integer;
    tZQI                : integer;
--    tZQCS               : integer;
--    USER_REFRESH        : string;
    TEMP_MON_EN         : string;
    WRLVL               : string;
    DEBUG_PORT          : string;
--    CAL_WIDTH           : integer;
    RANK_WIDTH          : integer;
    RANKS               : integer;
    ODT_WIDTH           : integer;
    ROW_WIDTH           : integer;
    ADDR_WIDTH          : integer;
    APP_DATA_WIDTH      : integer;
    APP_MASK_WIDTH      : integer;
    SIM_BYPASS_INIT_CAL : string;
    BYTE_LANES_B0       : std_logic_vector(3 downto 0);
    BYTE_LANES_B1       : std_logic_vector(3 downto 0);
    BYTE_LANES_B2       : std_logic_vector(3 downto 0);
    BYTE_LANES_B3       : std_logic_vector(3 downto 0);
    BYTE_LANES_B4       : std_logic_vector(3 downto 0);
    DATA_CTL_B0         : std_logic_vector(3 downto 0);
    DATA_CTL_B1         : std_logic_vector(3 downto 0);
    DATA_CTL_B2         : std_logic_vector(3 downto 0);
    DATA_CTL_B3         : std_logic_vector(3 downto 0);
    DATA_CTL_B4         : std_logic_vector(3 downto 0);
    PHY_0_BITLANES      : std_logic_vector(47 downto 0);
    PHY_1_BITLANES      : std_logic_vector(47 downto 0);
    PHY_2_BITLANES      : std_logic_vector(47 downto 0);
    CK_BYTE_MAP         : std_logic_vector(143 downto 0);
    ADDR_MAP            : std_logic_vector(191 downto 0);
    BANK_MAP            : std_logic_vector(35 downto 0);
    CAS_MAP             : std_logic_vector(11 downto 0);
    CKE_ODT_BYTE_MAP    : std_logic_vector(7 downto 0);
    CKE_MAP             : std_logic_vector(95 downto 0);
    ODT_MAP             : std_logic_vector(95 downto 0);
    CS_MAP              : std_logic_vector(119 downto 0);
    PARITY_MAP          : std_logic_vector(11 downto 0);
    RAS_MAP             : std_logic_vector(11 downto 0);
    WE_MAP              : std_logic_vector(11 downto 0);
    DQS_BYTE_MAP        : std_logic_vector(143 downto 0);
    DATA0_MAP           : std_logic_vector(95 downto 0);
    DATA1_MAP           : std_logic_vector(95 downto 0);
    DATA2_MAP           : std_logic_vector(95 downto 0);
    DATA3_MAP           : std_logic_vector(95 downto 0);
    DATA4_MAP           : std_logic_vector(95 downto 0);
    DATA5_MAP           : std_logic_vector(95 downto 0);
    DATA6_MAP           : std_logic_vector(95 downto 0);
    DATA7_MAP           : std_logic_vector(95 downto 0);
    DATA8_MAP           : std_logic_vector(95 downto 0);
    DATA9_MAP           : std_logic_vector(95 downto 0);
    DATA10_MAP          : std_logic_vector(95 downto 0);
    DATA11_MAP          : std_logic_vector(95 downto 0);
    DATA12_MAP          : std_logic_vector(95 downto 0);
    DATA13_MAP          : std_logic_vector(95 downto 0);
    DATA14_MAP          : std_logic_vector(95 downto 0);
    DATA15_MAP          : std_logic_vector(95 downto 0);
    DATA16_MAP          : std_logic_vector(95 downto 0);
    DATA17_MAP          : std_logic_vector(95 downto 0);
    MASK0_MAP           : std_logic_vector(107 downto 0);
    MASK1_MAP           : std_logic_vector(107 downto 0);
--    CALIB_ROW_ADD       : std_logic_vector(15 downto 0);
--    CALIB_COL_ADD       : std_logic_vector(11 downto 0);
--    CALIB_BA_ADD        : std_logic_vector(2 downto 0);
--    SLOT_0_CONFIG       : std_logic_vector(7 downto 0);
--    SLOT_1_CONFIG       : std_logic_vector(7 downto 0);
    MEM_ADDR_ORDER      : string;
--    STARVE_LIMIT        : integer;
--    MASTER_PHY_CTL      : integer;
    USE_CS_PORT         : integer;
    USE_DM_PORT         : integer;
    USE_ODT_PORT        : integer);
  port(
    clk                              : in    std_logic;
    clk_ref                          : in    std_logic;
    mem_refclk                       : in    std_logic;
    freq_refclk                      : in    std_logic;
    pll_lock                         : in    std_logic;
    sync_pulse                       : in    std_logic;
    rst                              : in    std_logic;
    rst_phaser_ref                   : in    std_logic;
    ref_dll_lock                     : out   std_logic;
    -- Memory interface ports
    ddr_dq                           : inout std_logic_vector(DQ_WIDTH-1 downto 0);
    ddr_dqs_n                        : inout std_logic_vector(DQS_WIDTH-1 downto 0);
    ddr_dqs                          : inout std_logic_vector(DQS_WIDTH-1 downto 0);
    ddr_addr                         : out   std_logic_vector(ROW_WIDTH-1 downto 0);
    ddr_ba                           : out   std_logic_vector(BANK_WIDTH-1 downto 0);
    ddr_cas_n                        : out   std_logic;
    ddr_ck_n                         : out   std_logic_vector(CK_WIDTH-1 downto 0);
    ddr_ck                           : out   std_logic_vector(CK_WIDTH-1 downto 0);
    ddr_cke                          : out   std_logic_vector(CKE_WIDTH-1 downto 0);
    ddr_cs_n                         : out   std_logic_vector(CS_WIDTH*nCS_PER_RANK-1 downto 0);
    ddr_dm                           : out   std_logic_vector(DM_WIDTH-1 downto 0);
    ddr_odt                          : out   std_logic_vector(RANKS-1 downto 0);
    ddr_ras_n                        : out   std_logic;
    ddr_reset_n                      : out   std_logic;
    ddr_parity                       : out   std_logic;
    ddr_we_n                         : out   std_logic;
    bank_mach_next                   : out   std_logic_vector(BM_CNT_WIDTH-1 downto 0);
    -- Application interface ports
    app_addr                         : in    std_logic_vector(ADDR_WIDTH-1 downto 0);
    app_cmd                          : in    std_logic_vector(2 downto 0);
    app_en                           : in    std_logic;
    app_hi_pri                       : in    std_logic;
    app_wdf_data                     : in    std_logic_vector(APP_DATA_WIDTH-1 downto 0);
    app_wdf_end                      : in    std_logic;
    app_wdf_mask                     : in    std_logic_vector(APP_MASK_WIDTH-1 downto 0);
    app_wdf_wren                     : in    std_logic;
    app_ecc_multiple_err             : out   std_logic_vector(2*nCK_PER_CLK-1 downto 0);
    app_rd_data                      : out   std_logic_vector(APP_DATA_WIDTH-1 downto 0);
    app_rd_data_end                  : out   std_logic;
    app_rd_data_valid                : out   std_logic;
    app_rdy                          : out   std_logic;
    app_wdf_rdy                      : out   std_logic;
    app_sr_req                       : in    std_logic;
    app_sr_active                    : out   std_logic;
    app_ref_req                      : in    std_logic;
    app_ref_ack                      : out   std_logic;
    app_zq_req                       : in    std_logic;
    app_zq_ack                       : out   std_logic;
    app_raw_not_ecc                  : in    std_logic_vector(2*nCK_PER_CLK-1 downto 0);
    app_correct_en_i                 : in    std_logic;
    -- Temperature ports
    device_temp                      : in    std_logic_vector(11 downto 0);
    -- Debug logic ports
    dbg_idel_up_all                  : in    std_logic;
    dbg_idel_down_all                : in    std_logic;
    dbg_idel_up_cpt                  : in    std_logic;
    dbg_idel_down_cpt                : in    std_logic;
    dbg_sel_idel_cpt                 : in    std_logic_vector(DQS_CNT_WIDTH-1 downto 0);
    dbg_sel_all_idel_cpt             : in    std_logic;
    dbg_sel_pi_incdec                : in    std_logic;
    dbg_sel_po_incdec                : in    std_logic;
    dbg_byte_sel                     : in    std_logic_vector(DQS_CNT_WIDTH downto 0);
    dbg_pi_f_inc                     : in    std_logic;
    dbg_pi_f_dec                     : in    std_logic;
    dbg_po_f_inc                     : in    std_logic;
    dbg_po_f_stg23_sel               : in    std_logic;
    dbg_po_f_dec                     : in    std_logic;
    dbg_cpt_tap_cnt                  : out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_dq_idelay_tap_cnt            : out   std_logic_vector(5*DQS_WIDTH*RANKS-1 downto 0);
    dbg_calib_top                    : out   std_logic_vector(255 downto 0);
    dbg_cpt_first_edge_cnt           : out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_cpt_second_edge_cnt          : out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_rd_data_offset               : out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_phy_rdlvl                    : out   std_logic_vector(255 downto 0);
    dbg_phy_wrcal                    : out   std_logic_vector(99 downto 0);
    dbg_final_po_fine_tap_cnt        : out   std_logic_vector(6*DQS_WIDTH-1 downto 0);
    dbg_final_po_coarse_tap_cnt      : out   std_logic_vector(3*DQS_WIDTH-1 downto 0);
    dbg_rd_data_edge_detect          : out   std_logic_vector(DQS_WIDTH-1 downto 0);
    dbg_rddata                       : out   std_logic_vector(2*nCK_PER_CLK*DQ_WIDTH-1 downto 0);
    dbg_rddata_valid                 : out   std_logic;
    dbg_rdlvl_done                   : out   std_logic_vector(1 downto 0);
    dbg_rdlvl_err                    : out   std_logic_vector(1 downto 0);
    dbg_rdlvl_start                  : out   std_logic_vector(1 downto 0);
    dbg_wrlvl_fine_tap_cnt           : out   std_logic_vector(6*DQS_WIDTH-1 downto 0);
    dbg_wrlvl_coarse_tap_cnt         : out   std_logic_vector(3*DQS_WIDTH-1 downto 0);
    dbg_tap_cnt_during_wrlvl         : out   std_logic_vector(5 downto 0);
    dbg_wl_edge_detect_valid         : out   std_logic;
    dbg_wrlvl_done                   : out   std_logic;
    dbg_wrlvl_err                    : out   std_logic;
    dbg_wrlvl_start                  : out   std_logic;
    dbg_phy_wrlvl                    : out   std_logic_vector(255 downto 0);
    dbg_phy_init                     : out   std_logic_vector(255 downto 0);
    dbg_prbs_rdlvl                   : out   std_logic_vector(255 downto 0);
    dbg_pi_counter_read_val          : out   std_logic_vector(5 downto 0);
    dbg_po_counter_read_val          : out   std_logic_vector(8 downto 0);
    dbg_pi_phaselock_start           : out   std_logic;
    dbg_pi_phaselocked_done          : out   std_logic;
    dbg_pi_phaselock_err             : out   std_logic;
    dbg_pi_phase_locked_phy4lanes    : out   std_logic_vector(11 downto 0);
    dbg_pi_dqsfound_start            : out   std_logic;
    dbg_pi_dqsfound_done             : out   std_logic;
    dbg_pi_dqsfound_err              : out   std_logic;
    dbg_pi_dqs_found_lanes_phy4lanes : out   std_logic_vector(11 downto 0);
    dbg_calib_rd_data_offset_1       : out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_calib_rd_data_offset_2       : out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_data_offset                  : out   std_logic_vector(5 downto 0);
    dbg_data_offset_1                : out   std_logic_vector(5 downto 0);
    dbg_data_offset_2                : out   std_logic_vector(5 downto 0);
    dbg_wrcal_start                  : out   std_logic;
    dbg_wrcal_done                   : out   std_logic;
    dbg_wrcal_err                    : out   std_logic;
    dbg_phy_oclkdelay_cal            : out   std_logic_vector(255 downto 0);
    dbg_oclkdelay_rd_data            : out   std_logic_vector(127 downto 0);
    dbg_oclkdelay_calib_start        : out   std_logic;
    dbg_oclkdelay_calib_done         : out   std_logic;
    dbg_dqs_found_cal                : out   std_logic_vector(255 downto 0);
    init_calib_complete              : out   std_logic);
  end component;


  -- Select speed bin constants
  -- mig_clocks_t type package ddr3_if_pkg
  constant MIG_CLOCKS : mig_clocks_t := clock;

  -- Constants for MIG simulation
  -- conv_sim_bypass_init_cal function package ddr3_if_pkg
  constant MIG_SIM_BYPASS_INIT_CAL : string  := conv_sim_bypass_init_cal(sim);
  constant MIG_SIMULATION          : string  := conv_simulation(sim);

  -- Constants for XADC
  -- conv_temp_mon_en function package ddr3_if_pkg
  constant MIG_TEMP_MON_EN         : string  := conv_temp_mon_en(sim);
  constant MIG_XADC_CLK_PERIOD     : integer := 5000;     -- Use 200 MHz IODELAYCTRL clock
  constant MIG_T_TEMP_SAMPLE       : integer := 10000000; -- sample every 10 us

  -- Only enable XADC for bank 0. Use XADC output for other banks.
  type mig_temp_array_t is array (0 to DDR3_BANKS-1) of string(1 to 8);
  constant MIG_TEMP_MON_CONTROL : mig_temp_array_t := ("INTERNAL","EXTERNAL");

  -- DDR3_BANK_ROW_WIDTH constant package adb3_target_inc_pkg
  -- conv_*_option function package ddr3_if_pkg
  constant MIG_T_RFC_OPTION   : integer := conv_t_rfc_option(DDR3_BANK_ROW_WIDTH);
  constant MIG_CL_OPTION      : integer := conv_cl_option(MIG_CLOCKS.tCK);
  constant MIG_CWL_OPTION     : integer := conv_cwl_option(MIG_CLOCKS.tCK);
  constant MIG_RTT_NOM_OPTION : string  := conv_rtt_nom_option(MIG_CLOCKS.tCK);
  constant MIG_T_CKE_OPTION   : integer := conv_t_cke_option(MIG_CLOCKS.tCK);
  constant MIG_T_FAW_OPTION   : integer := conv_t_faw_option(MIG_CLOCKS.tCK);
  constant MIG_T_RAS_OPTION   : integer := conv_t_ras_option(MIG_CLOCKS.tCK);
  constant MIG_T_RCD_OPTION   : integer := conv_t_rcd_option(MIG_CLOCKS.tCK);
  constant MIG_T_RP_OPTION    : integer := conv_t_rp_option(MIG_CLOCKS.tCK);
  constant MIG_T_RRD_OPTION   : integer := conv_t_rrd_option(MIG_CLOCKS.tCK);

  -- MIG_CLOCKS, MIG_COMMON constants package ddr3_if_pkg
  constant APP_DATA_WIDTH   : natural := 2 * MIG_CLOCKS.nCK_PER_CLK * MIG_COMMON.PAYLOAD_WIDTH;
  constant APP_MASK_WIDTH   : natural := APP_DATA_WIDTH / 8;
  constant DBG_RDDATA_WIDTH : natural := 2 * MIG_CLOCKS.nCK_PER_CLK * MIG_COMMON.DQ_WIDTH;

  constant APP_RAW_NOT_ECC : std_logic_vector(2*MIG_CLOCKS.nCK_PER_CLK-1 downto 0) := (others => '0');


  -- Indicates byte lanes used in an IO column.
  constant MIG_BYTE_LANES_BANK : mig_byte_lanes_t := (
    BYTE_LANES_B0 => B"1111",
    BYTE_LANES_B1 => B"0000",
    BYTE_LANES_B2 => B"0000",
    BYTE_LANES_B3 => B"0000",
    BYTE_LANES_B4 => B"0000");

  constant MIG_BYTE_LANES : byte_lanes_array_t(0 to DDR3_BANKS-1) := (others => MIG_BYTE_LANES_BANK);


  -- Indicates byte lane is data byte lane or control byte lane.
  -- '1' in a bit position indicates a data byte lane
  -- '0' in a bit position indicates a control byte lane
  constant MIG_DATA_CTL_BANK : mig_data_ctl_t := (
    DATA_CTL_B0 => X"6",
    DATA_CTL_B1 => X"0",
    DATA_CTL_B2 => X"0",
    DATA_CTL_B3 => X"0",
    DATA_CTL_B4 => X"0");

  constant MIG_DATA_CTL : data_ctl_array_t(0 to DDR3_BANKS-1) := (others => MIG_DATA_CTL_BANK);



  -- Select phy bitlanes constants for each bank using DDR3_BANK_ROW_WIDTH
  -- bitlanes_t, bitlanes_array_t, phy_bitlanes_t types package ddr3_if_pkg
  constant PHY_BITLANES_2G : phy_bitlanes_t := (
    PHY_0_BITLANES => X"FEE_3FE_3FE_FFF",  -- 2 Gib (DDR3-1600)
    PHY_1_BITLANES => X"000_000_000_000",
    PHY_2_BITLANES => X"000_000_000_000");

  constant PHY_BITLANES_4G : phy_bitlanes_t := (
    PHY_0_BITLANES => X"FFF_3FE_3FE_FFF",  -- 4 Gib (DDR3-800)
    PHY_1_BITLANES => X"000_000_000_000",
    PHY_2_BITLANES => X"000_000_000_000");

  constant SIZE_INDEX : natural range 0 to 1 := DDR3_BANK_ROW_WIDTH - 14; -- 2/4 Gib : 14/15 bit row width

  constant PHY_BITLANES : mig_phy_array_t(0 to 1) := (PHY_BITLANES_2G, PHY_BITLANES_4G);

  constant MIG_BITLANES : phy_bitlanes_t := PHY_BITLANES(SIZE_INDEX);


  -- Select mapping constants for each bank using DDR3_BANK_ROW_WIDTH
  -- addrmap_t, addrmap_array_t, mig_map_t types package ddr3_if_pkg
  constant ADDR_MAP_2G_BANK0 : addrmap_t := X"000_000_008_038_031_000_003_004_009_00A_00B_006_032_007_033_035";
  constant ADDR_MAP_4G_BANK0 : addrmap_t := X"000_030_008_038_031_000_003_004_009_00A_00B_006_032_007_033_035";

  constant ADDR_MAP_2G_BANK1 : addrmap_t := X"000_000_004_009_002_00B_000_008_005_036_007_001_006_00A_003_033";
  constant ADDR_MAP_4G_BANK1 : addrmap_t := X"000_030_004_009_002_00B_000_008_005_036_007_001_006_00A_003_033";

  constant ADDR_MAP_BANK0 : addrmap_array_t(0 to 1) := (ADDR_MAP_2G_BANK0, ADDR_MAP_4G_BANK0);
  constant ADDR_MAP_BANK1 : addrmap_array_t(0 to 1) := (ADDR_MAP_2G_BANK1, ADDR_MAP_4G_BANK1);

  constant MIG_MAP_BANK0 : mig_map_t :=
    (CK_BYTE_MAP      => X"00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_03",
     ADDR_MAP         => ADDR_MAP_BANK0(SIZE_INDEX),
     BANK_MAP         => X"037_039_001",
     CAS_MAP          => X"005",
     CKE_ODT_BYTE_MAP => X"00",
     CKE_MAP          => X"000_000_000_000_000_000_000_03A",
     ODT_MAP          => X"000_000_000_000_000_000_000_002",
     CS_MAP           => X"000_000_000_000_000_000_000_000_000_000",
     PARITY_MAP       => X"000",
     RAS_MAP          => X"036",
     WE_MAP           => X"03B",
     DQS_BYTE_MAP     => X"00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_02_01",
     DATA0_MAP        => X"011_018_016_017_015_012_014_013",
     DATA1_MAP        => X"025_024_023_028_026_027_022_021",
     DATA2_MAP        => X"000_000_000_000_000_000_000_000",
     DATA3_MAP        => X"000_000_000_000_000_000_000_000",
     DATA4_MAP        => X"000_000_000_000_000_000_000_000",
     DATA5_MAP        => X"000_000_000_000_000_000_000_000",
     DATA6_MAP        => X"000_000_000_000_000_000_000_000",
     DATA7_MAP        => X"000_000_000_000_000_000_000_000",
     DATA8_MAP        => X"000_000_000_000_000_000_000_000",
     DATA9_MAP        => X"000_000_000_000_000_000_000_000",
     DATA10_MAP       => X"000_000_000_000_000_000_000_000",
     DATA11_MAP       => X"000_000_000_000_000_000_000_000",
     DATA12_MAP       => X"000_000_000_000_000_000_000_000",
     DATA13_MAP       => X"000_000_000_000_000_000_000_000",
     DATA14_MAP       => X"000_000_000_000_000_000_000_000",
     DATA15_MAP       => X"000_000_000_000_000_000_000_000",
     DATA16_MAP       => X"000_000_000_000_000_000_000_000",
     DATA17_MAP       => X"000_000_000_000_000_000_000_000",
     MASK0_MAP        => X"000_000_000_000_000_000_000_029_019",
     MASK1_MAP        => X"000_000_000_000_000_000_000_000_000");

  constant MIG_MAP_BANK1 : mig_map_t :=
    (CK_BYTE_MAP      => X"00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_03",
     ADDR_MAP         => ADDR_MAP_BANK1(SIZE_INDEX),
     BANK_MAP         => X"038_032_031",
     CAS_MAP          => X"03A",
     CKE_ODT_BYTE_MAP => X"00",
     CKE_MAP          => X"000_000_000_000_000_000_000_037",
     ODT_MAP          => X"000_000_000_000_000_000_000_039",
     CS_MAP           => X"000_000_000_000_000_000_000_000_000_000",
     PARITY_MAP       => X"000",
     RAS_MAP          => X"03B",
     WE_MAP           => X"035",
     DQS_BYTE_MAP     => X"00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_00_02_01",
     DATA0_MAP        => X"014_012_017_018_015_013_011_016",
     DATA1_MAP        => X"021_028_023_025_027_026_024_022",
     DATA2_MAP        => X"000_000_000_000_000_000_000_000",
     DATA3_MAP        => X"000_000_000_000_000_000_000_000",
     DATA4_MAP        => X"000_000_000_000_000_000_000_000",
     DATA5_MAP        => X"000_000_000_000_000_000_000_000",
     DATA6_MAP        => X"000_000_000_000_000_000_000_000",
     DATA7_MAP        => X"000_000_000_000_000_000_000_000",
     DATA8_MAP        => X"000_000_000_000_000_000_000_000",
     DATA9_MAP        => X"000_000_000_000_000_000_000_000",
     DATA10_MAP       => X"000_000_000_000_000_000_000_000",
     DATA11_MAP       => X"000_000_000_000_000_000_000_000",
     DATA12_MAP       => X"000_000_000_000_000_000_000_000",
     DATA13_MAP       => X"000_000_000_000_000_000_000_000",
     DATA14_MAP       => X"000_000_000_000_000_000_000_000",
     DATA15_MAP       => X"000_000_000_000_000_000_000_000",
     DATA16_MAP       => X"000_000_000_000_000_000_000_000",
     DATA17_MAP       => X"000_000_000_000_000_000_000_000",
     MASK0_MAP        => X"000_000_000_000_000_000_000_029_019",
     MASK1_MAP        => X"000_000_000_000_000_000_000_000_000");

  constant MIG_MAP : mig_map_array_t(0 to DDR3_BANKS-1) := (MIG_MAP_BANK0, MIG_MAP_BANK1);


  -- infrastructure signals
  signal mig_ddr3_if_clk            : std_logic;
  signal mig_ddr3_if_mem_refclk     : std_logic;
  signal mig_ddr3_if_freq_refclk    : std_logic;
  signal mig_ddr3_if_pll_locked     : std_logic;
  signal mig_ddr3_if_sync_pulse     : std_logic;
  signal mig_ddr3_if_rst            : std_logic;
  signal mig_ddr3_if_rst_phaser_ref : std_logic;
  signal mig_ddr3_if_ref_dll_lock   : std_logic;

  -- memc_ui_top signals
  signal app_addr             : std_logic_vector(MIG_COMMON.ADDR_WIDTH-1 downto 0);
  signal app_cmd              : std_logic_vector(2 downto 0);
  signal app_en               : std_logic;
--  signal app_hi_pri           : std_logic;
  signal app_wdf_data         : std_logic_vector(APP_DATA_WIDTH-1 downto 0);
  signal app_wdf_end          : std_logic;
  signal app_wdf_mask         : std_logic_vector(APP_MASK_WIDTH-1 downto 0);
  signal app_wdf_wren         : std_logic;
--  signal app_ecc_multiple_err : std_logic_vector(2*MIG_CLOCKS.nCK_PER_CLK-1 downto 0);
  signal app_rd_data          : std_logic_vector(APP_DATA_WIDTH-1 downto 0);
--  signal app_rd_data_end      : std_logic;
  signal app_rd_data_valid    : std_logic;
  signal app_rdy              : std_logic;
  signal app_wdf_rdy          : std_logic;
--  signal app_sr_req           : std_logic;
--  signal app_sr_active        : std_logic;
--  signal app_ref_req          : std_logic;
--  signal app_ref_ack          : std_logic;
--  signal app_zq_req           : std_logic;
--  signal app_zq_ack           : std_logic;
--  signal app_raw_not_ecc      : std_logic_vector(2*MIG_CLOCKS.nCK_PER_CLK-1 downto 0);
--  signal app_correct_en_i     : std_logic;


  -- Temperature monitoring signals
  signal device_temp : std_logic_vector(11 downto 0);

  -- Calibration signals
  signal init_calib_complete : std_logic;

  -- Debug input signals
  signal dbg_idel_up_all      : std_logic := '0';
  signal dbg_idel_down_all    : std_logic := '0';
  signal dbg_idel_up_cpt      : std_logic := '0';
  signal dbg_idel_down_cpt    : std_logic := '0';
  signal dbg_sel_all_idel_cpt : std_logic := '0';
  signal dbg_sel_pi_incdec    : std_logic := '0';
  signal dbg_sel_po_incdec    : std_logic := '0';
  signal dbg_pi_f_inc         : std_logic := '0';
  signal dbg_pi_f_dec         : std_logic := '0';
  signal dbg_sel_idel_cpt     : std_logic_vector(MIG_COMMON.DQS_CNT_WIDTH-1 downto 0) := (others => '0');
  signal dbg_byte_sel         : std_logic_vector(MIG_COMMON.DQS_CNT_WIDTH downto 0) := (others => '0');
  signal dbg_po_f_inc         : std_logic := '0';
  signal dbg_po_f_stg23_sel   : std_logic := '0';
  signal dbg_po_f_dec         : std_logic := '0';

  -- Debug output signals
  signal dbg_cpt_tap_cnt                  : std_logic_vector(6*MIG_COMMON.DQS_WIDTH*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_dq_idelay_tap_cnt            : std_logic_vector(5*MIG_COMMON.DQS_WIDTH*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_calib_top                    : std_logic_vector(255 downto 0);
  signal dbg_cpt_first_edge_cnt           : std_logic_vector(6*MIG_COMMON.DQS_WIDTH*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_cpt_second_edge_cnt          : std_logic_vector(6*MIG_COMMON.DQS_WIDTH*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_rd_data_offset               : std_logic_vector(6*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_phy_rdlvl                    : std_logic_vector(255 downto 0);
  signal dbg_phy_wrcal                    : std_logic_vector(99 downto 0);
  signal dbg_final_po_fine_tap_cnt        : std_logic_vector(6*MIG_COMMON.DQS_WIDTH-1 downto 0);
  signal dbg_final_po_coarse_tap_cnt      : std_logic_vector(3*MIG_COMMON.DQS_WIDTH-1 downto 0);
  signal dbg_rd_data_edge_detect          : std_logic_vector(MIG_COMMON.DQS_WIDTH-1 downto 0);
  signal dbg_rddata                       : std_logic_vector(DBG_RDDATA_WIDTH-1 downto 0);
  signal dbg_rddata_valid                 : std_logic;
  signal dbg_rdlvl_done                   : std_logic_vector(1 downto 0);
  signal dbg_rdlvl_err                    : std_logic_vector(1 downto 0);
  signal dbg_rdlvl_start                  : std_logic_vector(1 downto 0);
  signal dbg_wrlvl_fine_tap_cnt           : std_logic_vector(6*MIG_COMMON.DQS_WIDTH-1 downto 0);
  signal dbg_wrlvl_coarse_tap_cnt         : std_logic_vector(3*MIG_COMMON.DQS_WIDTH-1 downto 0);
  signal dbg_tap_cnt_during_wrlvl         : std_logic_vector(5 downto 0);
  signal dbg_wl_edge_detect_valid         : std_logic;
  signal dbg_wrlvl_done                   : std_logic;
  signal dbg_wrlvl_err                    : std_logic;
  signal dbg_wrlvl_start                  : std_logic;
  signal dbg_phy_wrlvl                    : std_logic_vector(255 downto 0);
  signal dbg_phy_init                     : std_logic_vector(255 downto 0);
  signal dbg_prbs_rdlvl                   : std_logic_vector(255 downto 0);
  signal dbg_pi_counter_read_val          : std_logic_vector(5 downto 0);
  signal dbg_po_counter_read_val          : std_logic_vector(8 downto 0);
  signal dbg_pi_phaselock_start           : std_logic;
  signal dbg_pi_phaselocked_done          : std_logic;
  signal dbg_pi_phaselock_err             : std_logic;
  signal dbg_pi_phase_locked_phy4lanes    : std_logic_vector(11 downto 0);
  signal dbg_pi_dqsfound_start            : std_logic;
  signal dbg_pi_dqsfound_done             : std_logic;
  signal dbg_pi_dqsfound_err              : std_logic;
  signal dbg_pi_dqs_found_lanes_phy4lanes : std_logic_vector(11 downto 0);
  signal dbg_calib_rd_data_offset_1       : std_logic_vector(6*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_calib_rd_data_offset_2       : std_logic_vector(6*MIG_COMMON.RANKS-1 downto 0);
  signal dbg_data_offset                  : std_logic_vector(5 downto 0);
  signal dbg_data_offset_1                : std_logic_vector(5 downto 0);
  signal dbg_data_offset_2                : std_logic_vector(5 downto 0);
  signal dbg_wrcal_start                  : std_logic;
  signal dbg_wrcal_done                   : std_logic;
  signal dbg_wrcal_err                    : std_logic;
  signal dbg_phy_oclkdelay_cal            : std_logic_vector(255 downto 0);
  signal dbg_oclkdelay_rd_data            : std_logic_vector(127 downto 0);
  signal dbg_oclkdelay_calib_start        : std_logic;
  signal dbg_oclkdelay_calib_done         : std_logic;
  signal dbg_dqs_found_cal                : std_logic_vector(255 downto 0);

begin

  -- Check MIG interface used address widths
  assert not(DDR3_BANK_ROW_WIDTH /= MIG_COMMON.ROW_WIDTH)
    report "(ddr3_if_bank): MIG ROW_WIDTH /= target_inc_pkg DDR3_BANK_ROW_WIDTH"
    severity failure;

  -- Check MIG interface maximum address widths
  assert not(DDR3_BANK_ROW_WIDTH_MAX < MIG_COMMON.ROW_WIDTH)
    report "(ddr3_if_bank): MIG ROW_WIDTH > target_inc_pkg DDR3_BANK_ROW_WIDTH_MAX"
    severity failure;

  -- Should be Rank(1)+Bank(3)+Row(14/15)+Col(10) and remove Rank (-1) and convert to byte addressing (+2)
  assert not(DDR3_BYTE_ADDR_WIDTH /= (MIG_COMMON.ADDR_WIDTH - 1 + 2))
    report "(ddr3_if_bank): MIG ADDR_WIDTH /= target_inc_pkg DDR3_BYTE_ADDR_WIDTH"
    severity failure;

  assert not(DDR3_BANK_DATA_WIDTH /= MIG_COMMON.DQ_WIDTH)
    report "(ddr3_if_bank): MIG DQ_WIDTH /= target_inc_pkg DDR3_BANK_DATA_WIDTH"
    severity failure;


  -- Instantiate OCP to MIG block
  -- Disable app_sz output for MIG7
  adb3_ocp_ocp2ddr3_nb_i : adb3_ocp_ocp2ddr3_nb
  generic map(
    app_row_width     => MIG_COMMON.ROW_WIDTH,     -- natural
    app_col_width     => MIG_COMMON.COL_WIDTH,     -- natural
    app_bank_width    => MIG_COMMON.BANK_WIDTH,    -- natural
    app_addr_width    => MIG_COMMON.ADDR_WIDTH,    -- natural
    app_dqs_cnt_width => MIG_COMMON.DQS_CNT_WIDTH) -- natural := 2
  port map(
    -- OCP Interface
    ocp_rst           => ocp_rst,             -- in    std_logic
    ocp_clk           => ocp_clk,             -- in    std_logic
    ocp_m2s           => ocp_m2s,             -- in    adb3_ocp_m2sT
    ocp_s2m           => ocp_s2m,             -- out   adb3_ocp_s2mT
    -- MIG user interface
    mig_rst           => mig_ddr3_if_rst,     -- in    std_logic
    mig_clk           => mig_ddr3_if_clk,     -- in    std_logic
    phy_init_done     => init_calib_complete, -- in    std_logic
    app_rdy           => app_rdy,             -- in    std_logic
    app_wdf_rdy       => app_wdf_rdy,         -- in    std_logic
    app_rd_data       => app_rd_data,         -- in    adb3_ocp_data_s
    app_rd_data_valid => app_rd_data_valid,   -- in    std_logic
    app_en            => app_en,              -- out   std_logic
    app_cmd           => app_cmd,             -- out   std_logic_vector(2 downto 0)
    app_addr          => app_addr,            -- out   std_logic_vector(app_addr_width-1 downto 0);
    app_sz            => open,                -- out   std_logic
    app_wdf_wren      => app_wdf_wren,        -- out   std_logic
    app_wdf_data      => app_wdf_data,        -- out   adb3_ocp_data_s
    app_wdf_mask      => app_wdf_mask,        -- out   adb3_ocp_be_s
    app_wdf_end       => app_wdf_end,         -- out   std_logic
    -- Debug info
    debug_info        => ddr3_if_debug);      -- out   std_logic_vector(31 downto 0))


  -- Need component for Verilog memc_ui_top_std
  -- Instantiate DDR3 MIG bank core
  memc_ui_top_std_i : mig_7series_v1_8_memc_ui_top_std
  generic map(
    TCQ                   => MIG_COMMON.TCQ,
    ADDR_CMD_MODE         => MIG_COMMON.ADDR_CMD_MODE,
--    AL                    =>
    PAYLOAD_WIDTH         => MIG_COMMON.PAYLOAD_WIDTH,
    BANK_WIDTH            => MIG_COMMON.BANK_WIDTH,
    BM_CNT_WIDTH          => MIG_COMMON.BM_CNT_WIDTH,
    BURST_MODE            => MIG_COMMON.BURST_MODE,
--    BURST_TYPE            =>
--    CA_MIRROR             =>
    CK_WIDTH              => MIG_COMMON.CK_WIDTH,
    COL_WIDTH             => MIG_COMMON.COL_WIDTH,
--    CMD_PIPE_PLUS1        =>
    CS_WIDTH              => MIG_COMMON.CS_WIDTH,
    nCS_PER_RANK          => MIG_COMMON.nCS_PER_RANK,
    CKE_WIDTH             => MIG_COMMON.CKE_WIDTH,
    DATA_WIDTH            => MIG_COMMON.DATA_WIDTH,
    DATA_BUF_ADDR_WIDTH   => MIG_COMMON.DATA_BUF_ADDR_WIDTH,
--    DDR2_DQSN_ENABLE      =>
    DM_WIDTH              => MIG_COMMON.DM_WIDTH,
    DQ_CNT_WIDTH          => MIG_COMMON.DQ_CNT_WIDTH,
    DQ_WIDTH              => MIG_COMMON.DQ_WIDTH,
    DQS_CNT_WIDTH         => MIG_COMMON.DQS_CNT_WIDTH,
    DQS_WIDTH             => MIG_COMMON.DQS_WIDTH,
--    DRAM_TYPE             =>
--    DRAM_WIDTH            =>
--    ECC                   =>
--    ECC_WIDTH             =>
--    ECC_TEST              =>
    MC_ERR_ADDR_WIDTH     => MIG_COMMON.MC_ERR_ADDR_WIDTH,
    REFCLK_FREQ           => MIG_CLOCKS.REFCLK_FREQ,
--    nAL                   =>
--    nBANK_MACHS           =>
--    CKE_ODT_AUX           =>
    nCK_PER_CLK           => MIG_CLOCKS.nCK_PER_CLK,
    ORDERING              => MIG_COMMON.ORDERING,
    OUTPUT_DRV            => MIG_COMMON.OUTPUT_DRV,
--    IBUF_LPWR_MODE        =>
--    IODELAY_HP_MODE       =>
    DATA_IO_IDLE_PWRDWN   => MIG_COMMON.DATA_IO_IDLE_PWRDWN,
--    BANK_TYPE             =>
    DATA_IO_PRIM_TYPE     => MIG_COMMON.DATA_IO_PRIM_TYPE,
    IODELAY_GRP           => MIG_CLOCKS_COMMON.IODELAY_GRP,
    REG_CTRL              => MIG_COMMON.REG_CTRL,
    RTT_NOM               => MIG_RTT_NOM_OPTION,
    RTT_WR                => MIG_COMMON.RTT_WR,
    CL                    => MIG_CL_OPTION,
    CWL                   => MIG_CWL_OPTION,
    tCK                   => MIG_CLOCKS.tCK,
    tCKE                  => MIG_T_CKE_OPTION,
    tFAW                  => MIG_T_FAW_OPTION,
--    tPRDI                 =>
    tRAS                  => MIG_T_RAS_OPTION,
    tRCD                  => MIG_T_RCD_OPTION,
    tREFI                 => MIG_COMMON.tREFI,
    tRFC                  => MIG_T_RFC_OPTION,
    tRP                   => MIG_T_RP_OPTION,
    tRRD                  => MIG_T_RRD_OPTION,
--    tRTP                  =>
--    tWTR                  =>
    tZQI                  => MIG_COMMON.tZQI,
--    tZQCS                 =>
--    USER_REFRESH          =>
    TEMP_MON_EN           => MIG_TEMP_MON_EN,
    WRLVL                 => MIG_COMMON.WRLVL,
    DEBUG_PORT            => MIG_COMMON.DEBUG_PORT,
--    CAL_WIDTH             =>
    RANK_WIDTH            => MIG_COMMON.RANK_WIDTH,
    RANKS                 => MIG_COMMON.RANKS,
    ODT_WIDTH             => MIG_COMMON.ODT_WIDTH,
    ROW_WIDTH             => MIG_COMMON.ROW_WIDTH,
    ADDR_WIDTH            => MIG_COMMON.ADDR_WIDTH,
    APP_DATA_WIDTH        => APP_DATA_WIDTH,
    APP_MASK_WIDTH        => APP_MASK_WIDTH,
    SIM_BYPASS_INIT_CAL   => MIG_SIM_BYPASS_INIT_CAL,
    BYTE_LANES_B0         => MIG_BYTE_LANES(bank-DDR3_BANK0).BYTE_LANES_B0,
    BYTE_LANES_B1         => MIG_BYTE_LANES(bank-DDR3_BANK0).BYTE_LANES_B1,
    BYTE_LANES_B2         => MIG_BYTE_LANES(bank-DDR3_BANK0).BYTE_LANES_B2,
    BYTE_LANES_B3         => MIG_BYTE_LANES(bank-DDR3_BANK0).BYTE_LANES_B3,
    BYTE_LANES_B4         => MIG_BYTE_LANES(bank-DDR3_BANK0).BYTE_LANES_B4,
    DATA_CTL_B0           => MIG_DATA_CTL(bank-DDR3_BANK0).DATA_CTL_B0,
    DATA_CTL_B1           => MIG_DATA_CTL(bank-DDR3_BANK0).DATA_CTL_B1,
    DATA_CTL_B2           => MIG_DATA_CTL(bank-DDR3_BANK0).DATA_CTL_B2,
    DATA_CTL_B3           => MIG_DATA_CTL(bank-DDR3_BANK0).DATA_CTL_B3,
    DATA_CTL_B4           => MIG_DATA_CTL(bank-DDR3_BANK0).DATA_CTL_B4,
    PHY_0_BITLANES        => MIG_BITLANES.PHY_0_BITLANES,
    PHY_1_BITLANES        => MIG_BITLANES.PHY_1_BITLANES,
    PHY_2_BITLANES        => MIG_BITLANES.PHY_2_BITLANES,
    CK_BYTE_MAP           => MIG_MAP(bank-DDR3_BANK0).CK_BYTE_MAP,
    ADDR_MAP              => MIG_MAP(bank-DDR3_BANK0).ADDR_MAP,
    BANK_MAP              => MIG_MAP(bank-DDR3_BANK0).BANK_MAP,
    CAS_MAP               => MIG_MAP(bank-DDR3_BANK0).CAS_MAP,
    CKE_ODT_BYTE_MAP      => MIG_MAP(bank-DDR3_BANK0).CKE_ODT_BYTE_MAP,
    CKE_MAP               => MIG_MAP(bank-DDR3_BANK0).CKE_MAP,
    ODT_MAP               => MIG_MAP(bank-DDR3_BANK0).ODT_MAP,
    CS_MAP                => MIG_MAP(bank-DDR3_BANK0).CS_MAP,
    PARITY_MAP            => MIG_MAP(bank-DDR3_BANK0).PARITY_MAP,
    RAS_MAP               => MIG_MAP(bank-DDR3_BANK0).RAS_MAP,
    WE_MAP                => MIG_MAP(bank-DDR3_BANK0).WE_MAP,
    DQS_BYTE_MAP          => MIG_MAP(bank-DDR3_BANK0).DQS_BYTE_MAP,
    DATA0_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA0_MAP,
    DATA1_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA1_MAP,
    DATA2_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA2_MAP,
    DATA3_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA3_MAP,
    DATA4_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA4_MAP,
    DATA5_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA5_MAP,
    DATA6_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA6_MAP,
    DATA7_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA7_MAP,
    DATA8_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA8_MAP,
    DATA9_MAP             => MIG_MAP(bank-DDR3_BANK0).DATA9_MAP,
    DATA10_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA10_MAP,
    DATA11_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA11_MAP,
    DATA12_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA12_MAP,
    DATA13_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA13_MAP,
    DATA14_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA14_MAP,
    DATA15_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA15_MAP,
    DATA16_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA16_MAP,
    DATA17_MAP            => MIG_MAP(bank-DDR3_BANK0).DATA17_MAP,
    MASK0_MAP             => MIG_MAP(bank-DDR3_BANK0).MASK0_MAP,
    MASK1_MAP             => MIG_MAP(bank-DDR3_BANK0).MASK1_MAP,
--    CALIB_ROW_ADD         =>
--    CALIB_COL_ADD         =>
--    CALIB_BA_ADD          =>
--    SLOT_0_CONFIG         =>
--    SLOT_1_CONFIG         =>
    MEM_ADDR_ORDER        => MIG_COMMON.MEM_ADDR_ORDER,
--    STARVE_LIMIT          =>
    USE_CS_PORT           => MIG_COMMON.USE_CS_PORT,
    USE_DM_PORT           => MIG_COMMON.USE_DM_PORT,
    USE_ODT_PORT          => MIG_COMMON.USE_ODT_PORT)
--    MASTER_PHY_CTL        => )
  port map(
    clk                              => mig_ddr3_if_clk,                                  -- in    std_logic;
    clk_ref                          => ddr3_ref_clk,                                     -- in    std_logic;
    mem_refclk                       => mig_ddr3_if_mem_refclk,                           -- in    std_logic;
    freq_refclk                      => mig_ddr3_if_freq_refclk,                          -- in    std_logic;
    pll_lock                         => mig_ddr3_if_pll_locked,                           -- in    std_logic;
    sync_pulse                       => mig_ddr3_if_sync_pulse,                           -- in    std_logic;
    rst                              => mig_ddr3_if_rst,                                  -- in    std_logic;
    rst_phaser_ref                   => mig_ddr3_if_rst_phaser_ref,                       -- in    std_logic;
    ref_dll_lock                     => mig_ddr3_if_ref_dll_lock,                         -- out   std_logic;
    -- Memory interface ports
    ddr_dq                           => ddr3_data_inout.dq,                               -- inout std_logic_vector(DQ_WIDTH-1 downto 0);
    ddr_dqs_n                        => ddr3_data_inout.dqs_n,                            -- inout std_logic_vector(DQS_WIDTH-1 downto 0);
    ddr_dqs                          => ddr3_data_inout.dqs_p,                            -- inout std_logic_vector(DQS_WIDTH-1 downto 0);
    ddr_addr                         => ddr3_addr_out.a(MIG_COMMON.ROW_WIDTH-1 downto 0), -- out   std_logic_vector(ROW_WIDTH-1 downto 0);
    ddr_ba                           => ddr3_ctrl_out.ba,                                 -- out   std_logic_vector(BANK_WIDTH-1 downto 0);
    ddr_cas_n                        => ddr3_ctrl_out.cas_l,                              -- out   std_logic;
    ddr_ck_n                         => ddr3_clk_out.clk_n,                               -- out   std_logic_vector(CK_WIDTH-1 downto 0);
    ddr_ck                           => ddr3_clk_out.clk_p,                               -- out   std_logic_vector(CK_WIDTH-1 downto 0);
    ddr_cke                          => ddr3_ctrl_out.cke,                                -- out   std_logic_vector(CKE_WIDTH-1 downto 0);
    ddr_cs_n                         => open,                                             -- out   std_logic_vector(CS_WIDTH*nCS_PER_RANK-1 downto 0);
    ddr_dm                           => ddr3_ctrl_out.dm,                                 -- out   std_logic_vector(DM_WIDTH-1 downto 0);
    ddr_odt                          => ddr3_ctrl_out.odt,                                -- out   std_logic_vector(RANKS-1 downto 0);
    ddr_ras_n                        => ddr3_ctrl_out.ras_l,                              -- out   std_logic;
    ddr_reset_n                      => ddr3_ctrl_out.reset_l,                            -- out   std_logic;
    ddr_parity                       => open,                                             -- out   std_logic;
    ddr_we_n                         => ddr3_ctrl_out.we_l,                               -- out   std_logic;
    bank_mach_next                   => open,                                             -- out   std_logic_vector(BM_CNT_WIDTH-1 downto 0);
    -- Calibration
    init_calib_complete              => init_calib_complete,                              -- out   std_logic);
    -- Application interface ports
    app_addr                         => app_addr,                                         -- in    std_logic_vector(ADDR_WIDTH-1 downto 0);
    app_cmd                          => app_cmd,                                          -- in    std_logic_vector(2 downto 0);
    app_en                           => app_en,                                           -- in    std_logic;
    app_hi_pri                       => '0',                                              -- in    std_logic;
    app_wdf_data                     => app_wdf_data,                                     -- in    std_logic_vector(APP_DATA_WIDTH-1 downto 0);
    app_wdf_end                      => app_wdf_end,                                      -- in    std_logic;
    app_wdf_mask                     => app_wdf_mask,                                     -- in    std_logic_vector(APP_MASK_WIDTH-1 downto 0);
    app_wdf_wren                     => app_wdf_wren,                                     -- in    std_logic;
    app_ecc_multiple_err             => open,                                             -- out   std_logic_vector(2*nCK_PER_CLK-1 downto 0);
    app_rd_data                      => app_rd_data,                                      -- out   std_logic_vector(APP_DATA_WIDTH-1 downto 0);
    app_rd_data_end                  => open,                                             -- out   std_logic;
    app_rd_data_valid                => app_rd_data_valid,                                -- out   std_logic;
    app_rdy                          => app_rdy,                                          -- out   std_logic;
    app_wdf_rdy                      => app_wdf_rdy,                                      -- out   std_logic;
    app_sr_req                       => '0',                                              -- in    std_logic;
    app_sr_active                    => open,                                             -- out   std_logic;
    app_ref_req                      => '0',                                              -- in    std_logic;
    app_ref_ack                      => open,                                             -- out   std_logic;
    app_zq_req                       => '0',                                              -- in    std_logic;
    app_zq_ack                       => open,                                             -- out   std_logic;
    app_raw_not_ecc                  => APP_RAW_NOT_ECC,                                  -- in    std_logic_vector(2*nCK_PER_CLK-1 downto 0);
    app_correct_en_i                 => '0',                                              -- in    std_logic;
    -- Temperature ports
    device_temp                      => device_temp,                                      -- in    std_logic_vector(11 downto 0);
    -- Debug logic ports
    dbg_idel_up_all                  => dbg_idel_up_all,                                  -- in    std_logic;
    dbg_idel_down_all                => dbg_idel_down_all,                                -- in    std_logic;
    dbg_idel_up_cpt                  => dbg_idel_up_cpt,                                  -- in    std_logic;
    dbg_idel_down_cpt                => dbg_idel_down_cpt,                                -- in    std_logic;
    dbg_sel_idel_cpt                 => dbg_sel_idel_cpt,                                 -- in    std_logic_vector(DQS_CNT_WIDTH-1 downto 0);
    dbg_sel_all_idel_cpt             => dbg_sel_all_idel_cpt,                             -- in    std_logic;
    dbg_sel_pi_incdec                => dbg_sel_pi_incdec,                                -- in    std_logic;
    dbg_sel_po_incdec                => dbg_sel_po_incdec,                                -- in    std_logic;
    dbg_byte_sel                     => dbg_byte_sel,                                     -- in    std_logic_vector(DQS_CNT_WIDTH downto 0);
    dbg_pi_f_inc                     => dbg_pi_f_inc,                                     -- in    std_logic;
    dbg_pi_f_dec                     => dbg_pi_f_dec,                                     -- in    std_logic;
    dbg_po_f_inc                     => dbg_po_f_inc,                                     -- in    std_logic;
    dbg_po_f_stg23_sel               => dbg_po_f_stg23_sel,                               -- in    std_logic;
    dbg_po_f_dec                     => dbg_po_f_dec,                                     -- in    std_logic;
    dbg_cpt_tap_cnt                  => dbg_cpt_tap_cnt,                                  -- out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_dq_idelay_tap_cnt            => dbg_dq_idelay_tap_cnt,                            -- out   std_logic_vector(5*DQS_WIDTH*RANKS-1 downto 0);
    dbg_calib_top                    => dbg_calib_top,                                    -- out   std_logic_vector(255 downto 0);
    dbg_cpt_first_edge_cnt           => dbg_cpt_first_edge_cnt,                           -- out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_cpt_second_edge_cnt          => dbg_cpt_second_edge_cnt,                          -- out   std_logic_vector(6*DQS_WIDTH*RANKS-1 downto 0);
    dbg_rd_data_offset               => dbg_rd_data_offset,                               -- out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_phy_rdlvl                    => dbg_phy_rdlvl,                                    -- out   std_logic_vector(255 downto 0);
    dbg_phy_wrcal                    => dbg_phy_wrcal,                                    -- out   std_logic_vector(99 downto 0);
    dbg_final_po_fine_tap_cnt        => dbg_final_po_fine_tap_cnt,                        -- out   std_logic_vector(6*DQS_WIDTH-1 downto 0);
    dbg_final_po_coarse_tap_cnt      => dbg_final_po_coarse_tap_cnt,                      -- out   std_logic_vector(3*DQS_WIDTH-1 downto 0);
    dbg_rd_data_edge_detect          => dbg_rd_data_edge_detect,                          -- out   std_logic_vector(DQS_WIDTH-1 downto 0);
    dbg_rddata                       => dbg_rddata,                                       -- out   std_logic_vector(2*nCK_PER_CLK*DQ_WIDTH-1 downto 0);
    dbg_rddata_valid                 => dbg_rddata_valid,                                 -- out   std_logic;
    dbg_rdlvl_done                   => dbg_rdlvl_done,                                   -- out   std_logic_vector(1 downto 0);
    dbg_rdlvl_err                    => dbg_rdlvl_err,                                    -- out   std_logic_vector(1 downto 0);
    dbg_rdlvl_start                  => dbg_rdlvl_start,                                  -- out   std_logic_vector(1 downto 0);
    dbg_wrlvl_fine_tap_cnt           => dbg_wrlvl_fine_tap_cnt,                           -- out   std_logic_vector(6*DQS_WIDTH-1 downto 0);
    dbg_wrlvl_coarse_tap_cnt         => dbg_wrlvl_coarse_tap_cnt,                         -- out   std_logic_vector(3*DQS_WIDTH-1 downto 0);
    dbg_tap_cnt_during_wrlvl         => dbg_tap_cnt_during_wrlvl,                         -- out   std_logic_vector(5 downto 0);
    dbg_wl_edge_detect_valid         => dbg_wl_edge_detect_valid,                         -- out   std_logic;
    dbg_wrlvl_done                   => dbg_wrlvl_done,                                   -- out   std_logic;
    dbg_wrlvl_err                    => dbg_wrlvl_err,                                    -- out   std_logic;
    dbg_wrlvl_start                  => dbg_wrlvl_start,                                  -- out   std_logic;
    dbg_phy_wrlvl                    => dbg_phy_wrlvl,                                    -- out   std_logic_vector(255 downto 0);
    dbg_phy_init                     => dbg_phy_init,                                     -- out   std_logic_vector(255 downto 0);
    dbg_prbs_rdlvl                   => dbg_prbs_rdlvl,                                   -- out   std_logic_vector(255 downto 0);
    dbg_pi_counter_read_val          => dbg_pi_counter_read_val,                          -- out   std_logic_vector(5 downto 0);
    dbg_po_counter_read_val          => dbg_po_counter_read_val,                          -- out   std_logic_vector(8 downto 0);
    dbg_pi_phaselock_start           => dbg_pi_phaselock_start ,                          -- out   std_logic;
    dbg_pi_phaselocked_done          => dbg_pi_phaselocked_done,                          -- out   std_logic;
    dbg_pi_phaselock_err             => dbg_pi_phaselock_err,                             -- out   std_logic;
    dbg_pi_phase_locked_phy4lanes    => dbg_pi_phase_locked_phy4lanes,                    -- out   std_logic_vector(11 downto 0);
    dbg_pi_dqsfound_start            => dbg_pi_dqsfound_start,                            -- out   std_logic;
    dbg_pi_dqsfound_done             => dbg_pi_dqsfound_done,                             -- out   std_logic;
    dbg_pi_dqsfound_err              => dbg_pi_dqsfound_err,                              -- out   std_logic;
    dbg_pi_dqs_found_lanes_phy4lanes => dbg_pi_dqs_found_lanes_phy4lanes,                 -- out   std_logic_vector(11 downto 0);
    dbg_calib_rd_data_offset_1       => dbg_calib_rd_data_offset_1,                       -- out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_calib_rd_data_offset_2       => dbg_calib_rd_data_offset_2,                       -- out   std_logic_vector(6*RANKS-1 downto 0);
    dbg_data_offset                  => dbg_data_offset,                                  -- out   std_logic_vector(5 downto 0);
    dbg_data_offset_1                => dbg_data_offset_1,                                -- out   std_logic_vector(5 downto 0);
    dbg_data_offset_2                => dbg_data_offset_2,                                -- out   std_logic_vector(5 downto 0);
    dbg_wrcal_start                  => dbg_wrcal_start,                                  -- out   std_logic;
    dbg_wrcal_done                   => dbg_wrcal_done,                                   -- out   std_logic;
    dbg_wrcal_err                    => dbg_wrcal_err,                                    -- out   std_logic;
    dbg_phy_oclkdelay_cal            => dbg_phy_oclkdelay_cal,                            -- out   std_logic_vector(255 downto 0);
    dbg_oclkdelay_rd_data            => dbg_oclkdelay_rd_data,                            -- out   std_logic_vector(127 downto 0);
    dbg_oclkdelay_calib_start        => dbg_oclkdelay_calib_start,                        -- out   std_logic;
    dbg_oclkdelay_calib_done         => dbg_oclkdelay_calib_done,                         -- out   std_logic;
    dbg_dqs_found_cal                => dbg_dqs_found_cal);                               -- out   std_logic_vector(255 downto 0);

  -- Tie unused address bits low
  addr_unused_g : if (DDR3_BANK_ROW_WIDTH_MAX > MIG_COMMON.ROW_WIDTH) generate
    ddr3_addr_out.a(DDR3_BANK_ROW_WIDTH_MAX-1 downto MIG_COMMON.ROW_WIDTH) <= (others => '0');
  end generate;


  -- DDR3 MIG clock generation
  infrastructure_i : mig_7series_v1_8_infrastructure
  generic map(
    SIMULATION     => MIG_SIMULATION,                -- string
    TCQ            => MIG_COMMON.TCQ,                -- integer
    nCK_PER_CLK    => MIG_CLOCKS.nCK_PER_CLK,        -- integer
    CLKIN_PERIOD   => MIG_CLOCKS.CLKIN_PERIOD,       -- integer
    CLKFBOUT_MULT  => MIG_CLOCKS.CLKFBOUT_MULT,      -- integer
    DIVCLK_DIVIDE  => MIG_CLOCKS.DIVCLK_DIVIDE,      -- integer
    CLKOUT0_PHASE  => MIG_CLOCKS.CLKOUT0_PHASE,      -- real
    CLKOUT0_DIVIDE => MIG_CLOCKS.CLKOUT0_DIVIDE,     -- integer
    CLKOUT1_DIVIDE => MIG_CLOCKS.CLKOUT1_DIVIDE,     -- integer
    CLKOUT2_DIVIDE => MIG_CLOCKS.CLKOUT2_DIVIDE,     -- integer
    CLKOUT3_DIVIDE => MIG_CLOCKS.CLKOUT3_DIVIDE,     -- integer
    RST_ACT_LOW    => MIG_CLOCKS_COMMON.RST_ACT_LOW) -- integer
  port map(
    clk              => mig_ddr3_if_clk,              -- out    std_logic;
    mem_refclk       => mig_ddr3_if_mem_refclk,       -- out    std_logic;
    freq_refclk      => mig_ddr3_if_freq_refclk,      -- out    std_logic;
    sync_pulse       => mig_ddr3_if_sync_pulse,       -- out    std_logic;
    auxout_clk       => open,                         -- out    std_logic;
    pll_locked       => mig_ddr3_if_pll_locked,       -- out    std_logic;
    rst_phaser_ref   => mig_ddr3_if_rst_phaser_ref,   -- out    std_logic;
    rstdiv0          => mig_ddr3_if_rst,              -- out    std_logic;
    mmcm_clk         => ddr3_clk,                     -- in     std_logic;
    sys_rst          => ddr3_rst,                     -- in     std_logic;
    iodelay_ctrl_rdy => ddr3_iodelay_ctrl_rdy,        -- in     std_logic;
    ref_dll_lock     => mig_ddr3_if_ref_dll_lock);    -- in     std_logic;


  -- Instantiate XADC during synthesis
  tempmon_on_g : if (MIG_TEMP_MON_EN = "ON") generate
    tempmon_i : mig_7series_v1_8_tempmon
    generic map(
      TCQ              => MIG_COMMON.TCQ,                        -- integer
      TEMP_MON_CONTROL => MIG_TEMP_MON_CONTROL(bank-DDR3_BANK0), -- string
      XADC_CLK_PERIOD  => MIG_XADC_CLK_PERIOD,                   -- integer
      tTEMPSAMPLE      => MIG_T_TEMP_SAMPLE)                     -- integer
    port map(
      clk           => mig_ddr3_if_clk,   -- in     std_logic
      xadc_clk      => ddr3_ref_clk,      -- in     std_logic
      rst           => mig_ddr3_if_rst,   -- in     std_logic
      device_temp_i => ddr3_xadc_temp_in, -- in     std_logic_vector(11 downto 0)
      device_temp   => device_temp);      -- out    std_logic_vector(11 downto 0)

     ddr3_xadc_temp_out <= device_temp;
  end generate;

  -- Do not use XADC during simulation
  tempmon_off_g : if (MIG_TEMP_MON_EN /= "ON") generate
    ddr3_xadc_temp_out <= (others => '0');
    device_temp        <= (others => '0');
  end generate;


  process(mig_ddr3_if_clk)
  begin
    if rising_edge(mig_ddr3_if_clk) then
      -- Connect ready
      ddr3_if_rdy <= init_calib_complete;
    end if;
  end process;

  -- Connect status
  ddr3_if_stat(0) <= dbg_wrlvl_done;
  ddr3_if_stat(1) <= dbg_rdlvl_done(0);
  ddr3_if_stat(2) <= dbg_rdlvl_done(1);
  ddr3_if_stat(3) <= init_calib_complete;
  -- Connect error
  ddr3_if_err(0)  <= dbg_wrlvl_err;
  ddr3_if_err(1)  <= dbg_rdlvl_err(0);
  ddr3_if_err(2)  <= dbg_rdlvl_err(1);
  ddr3_if_err(3)  <= mig_ddr3_if_rst;

end;
