#
# Script for checking Xilinx ISE version currently in use
#
# Usage: xtclsh check_ise_ver.tcl <min> <max>
# eg.
#        xtclsh check_ise_ver.tcl 14.2 14.2
#        xtclsh check_ise_ver.tcl 14.2 *
#        xtclsh check_ise_ver.tcl * 14.2
#

#
proc show_usage { } {
  global mig_ver
  global ise_1st
  puts "Usage:"
  puts ""
  puts "  xtclsh check_ise_ver.tcl <min> <max> { <min> <max> }"
  puts ""
  puts "Note:"
  puts ""
  puts "  '*' may be used as a wildcard for <min> or <max>."
}

proc check_range { } {
  global ise_first
  global ise_last
  global ise_wild
  global ise_num
  global ise_pass
  #
  # Perform checking of version
  #
  if { [string compare ${ise_first} ${ise_wild}] != 0 && [string compare ${ise_last} ${ise_wild}] != 0 } {
    if {${ise_num} < ${ise_first} || ${ise_num} > ${ise_last}} {
      if {${ise_first} == ${ise_last}} {
        puts "Fail: ISE version ${ise_first} is required."
      } else {
        puts "Fail: ISE version in range ${ise_first} to ${ise_last} is required."
      }
    } else {
      set ise_pass 1
      if {${ise_first} == ${ise_last}} {
        puts "Pass: ISE version ${ise_first} is satisfied."
      } else {
        puts "Pass: ISE version in range ${ise_first} to ${ise_last} is satisfied."
      }
    }
  #
  } else {
    if {[string compare ${ise_first} ${ise_wild}] != 0} {
      if {${ise_num} < ${ise_first}} {
        puts "Fail: ISE version ${ise_first} or later is required."
      } else {
        set ise_pass 1
        puts "Pass: ISE version ${ise_first} or later is satisfied."
      }
    } else {
      puts "Note: ISE first is unchecked."
    }
  #
    if {[string compare ${ise_last} ${ise_wild}] != 0} {
      if {${ise_num} > ${ise_last}} {
        puts "Fail: ISE version ${ise_last} or earlier is required."
      } else {
        set ise_pass 1
        puts "Pass: ISE version ${ise_last} or earlier is satisfied."
      }
    } else {
      puts "Note: ISE last is unchecked."
    }
  }
}


#
# Check parameters (valid in sets of 2)
#
if { ${argc} != 0 && ${argc} != 2 && ${argc} != 4 } {
  puts "ERROR: Invalid option '${argv}'"
  show_usage
  exit -1
}


#
#
# Get ISE version from fileset.txt
if {[info exists env(XILINX)]} {
  set _Native [ file nativename $env(XILINX) ]
  set _ISE_Path ${_Native}/fileset.txt
} else {
  puts "ERROR: %XILINX% environmental variable is not set."
  exit -1
}
#
#
# Get read fileset.txt
if {[file readable ${_ISE_Path}]} {
  set ise_file [open "${_ISE_Path}" r]
  set file_data [read $ise_file]
  close $ise_file
} else {
  puts "ERROR: Cannot read file '${_ISE_Path}'."
  exit -1
}
#
#
# search fileset.txt
puts ""
puts "Searching ${_ISE_Path} for ISE version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase " version=(.*)" ${line} result ise_num]} {
    incr found
#    puts "  ($found): ${result}, ${ise_num}"
  }
}

if {${found} eq 1} {
  puts "Found ISE version ${ise_num}"
} else {
  puts "Error: Found more than 1 (${found}) version entries in ${_ISE_Path}"
  exit -1
}


#
# Exit if info only option or odd number of args
#
if { $argc == 0 } {
  exit 0
}

set ise_wild *
set ise_pass 0

if { $argc == 2 || $argc == 4 } {
  set ise_first [ lindex $argv 0 ];
  set ise_last  [ lindex $argv 1 ];
  check_range
}

if { $argc == 4 } {
  set ise_first [ lindex $argv 2 ];
  set ise_last  [ lindex $argv 3 ];
  check_range
}

if { $ise_pass == 0 } {
  puts "Error: ISE version in use (${ise_num}) is not in required range."
  puts "Please refer to ADM-XRC Gen 3 User Guide."
  puts ""
  exit -1;
} else {
  puts "ISE varsion in use (${ise_num}) is in required range."
  puts ""
}
