# Generate date and time package
if { ! [ file exists planahead ] } {
  file mkdir planahead
}
exec -- xtclsh gen_today_pkg.tcl > planahead/today_pkg_${model}_${device}.vhd

# MIG DDR3 SDRAM interface sources (for synthesis & simulation)
set mig_ver "1_8"
set var_mig_path "ddr3_sdram_if/${model}/mig7_v${mig_ver}/rtl"
exec -- xtclsh ../../common/ddr3_sdram_if/check_ddr3_if_mig7_v${mig_ver}.tcl ${model}

# Select transceiver type
if {${device} == "7vx330t"} {
  set gt_type "h"
} elseif {${device} == "7vx485t"} {
  set gt_type "x"
} elseif {${device} == "7vx690t"} {
  set gt_type "h"
} elseif {${device} == "7v585t"} {
  set gt_type "x"
} else {
  puts "${design}-${model}.tcl: ${device} is not supported for model ${model}"
  return -1
}

set src_mig_ddr3 [ list \
  "../../common/${var_mig_path}/ui/mig_7series_v${mig_ver}_ui_cmd.v" \
  "../../common/${var_mig_path}/ui/mig_7series_v${mig_ver}_ui_rd_data.v" \
  "../../common/${var_mig_path}/ui/mig_7series_v${mig_ver}_ui_wr_data.v" \
  "../../common/${var_mig_path}/ui/mig_7series_v${mig_ver}_ui_top.v" \
  "../../common/${var_mig_path}/ecc/mig_7series_v${mig_ver}_ecc_merge_enc.v" \
  "../../common/${var_mig_path}/ecc/mig_7series_v${mig_ver}_ecc_dec_fix.v" \
  "../../common/${var_mig_path}/ecc/mig_7series_v${mig_ver}_ecc_buf.v" \
  "../../common/${var_mig_path}/ecc/mig_7series_v${mig_ver}_ecc_gen.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_byte_group_io.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_byte_lane.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_tempmon.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_calib_top.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_if_post_fifo.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_mc_phy.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_mc_phy_wrapper.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_of_pre_fifo.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_4lanes.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_ck_addr_cmd_delay.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_dqs_found_cal.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_dqs_found_cal_hr.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_init.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_oclkdelay_cal.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_prbs_rdlvl.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_rdlvl.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_wrcal.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_wrlvl.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_prbs_gen.v" \
  "../../common/${var_mig_path}/phy/mig_7series_v${mig_ver}_ddr_phy_top.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_arb_mux.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_arb_row_col.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_arb_select.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_cntrl.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_common.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_compare.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_mach.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_queue.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_bank_state.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_col_mach.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_rank_cntrl.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_rank_common.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_rank_mach.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_round_robin_arb.v" \
  "../../common/${var_mig_path}/controller/mig_7series_v${mig_ver}_mc.v" \
  "../../common/${var_mig_path}/clocking/mig_7series_v${mig_ver}_infrastructure.v" \
  "../../common/${var_mig_path}/clocking/mig_7series_v${mig_ver}_tempmon.v" \
  "../../common/${var_mig_path}/ip_top/mig_7series_v${mig_ver}_mem_intfc.v" \
  "../../common/${var_mig_path}/ip_top/mig_7series_v${mig_ver}_memc_ui_top_std.v" \
]

# Synthesis sources
set src_syn ${src_mig_ddr3}
lappend src_syn [ list \
  "../../common/adb3_ocp/adb3_ocp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_comp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_cross_clk_dom.vhd" \
  "../../common/adb3_ocp/adb3_ocp_retime_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_b.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_mux_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_ocp2ddr3_nb_s7.vhd" \
  "../../common/adb3_target/${model}/adb3_target_inc_pkg_syn_ngc_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_target/mptl_if_target_wrap_${model}${gt_type}.vhd" \
  "../../common/ddr3_sdram_if/ddr3_if_pkg_v7.vhd" \
  "../../common/ddr3_sdram_if/${model}/mig7_v${mig_ver}/ddr3_if_bank_${model}_mig7_v${mig_ver}.vhd" \
  "../../common/mem_apps/blk_mem_test.vhd" \
  "../../common/chipscope/blk_chipscope.vhd" \
  "planahead/today_pkg_${model}_${device}.vhd" \
  "common/uber_pkg_s7.vhd" \
  "${model}/blk_clocks_${model}.vhd" \
  "common/blk_ds_simple_test.vhd" \
  "common/blk_clock_freq.vhd" \
  "${model}/blk_ds_clk_read_${model}.vhd" \
  "${model}/blk_ds_io_test_${model}.vhd" \
  "common/blk_ds_int_test.vhd" \
  "common/blk_ds_mem_reg.vhd" \
  "common/blk_ds_info.vhd" \
  "common/blk_direct_slave.vhd" \
  "common/bram_single_wrap_s7.vhd" \
  "common/blk_bram.vhd" \
  "common/blk_dma_switch.vhd" \
  "common/blk_mem_app.vhd" \
  "${model}/blk_mem_if_${model}.vhd" \
  "common/uber_s7.vhd" \
]

# Constraints
set constraint [ list \
  "${model}/${design}-${model}-${device}.ucf" \
  "${model}/${design}-${model}${gt_type}.xcf" \
]

# Core search path for synthesis; relative to the temporary XST script file
# created by PlanAhead during synthesis. It's possible that Xilinx could break
# this in future by changing the structure of the tree generated by PlanAhead.
set core_dir [ list \
  "../../../../../common/adb3_target/${model}/mptl_target" \
]

# Simulation sources for OCP-only simulation
set src_sim_ocp ${src_mig_ddr3}
lappend src_sim_ocp [ list \
  "../../common/adb3_ocp/adb3_ocp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_comp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_cross_clk_dom.vhd" \
  "../../common/adb3_ocp/adb3_ocp_retime_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_b.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_mux_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_ocp2ddr3_nb_s7.vhd" \
  "../../common/adb3_target/${model}/adb3_target_inc_pkg_sim_ocp_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_target/mptl_if_target_wrap_sim_${model}.vhd" \
  "../../common/ddr3_sdram_if/ddr3_if_pkg_v7.vhd" \
  "../../common/ddr3_sdram_if/${model}/mig7_v${mig_ver}/ddr3_if_bank_${model}_mig7_v${mig_ver}.vhd" \
  "../../common/mem_apps/blk_mem_test.vhd" \
  "../../common/chipscope/blk_chipscope_sim.vhd" \
  "planahead/today_pkg_${model}_${device}.vhd" \
  "common/uber_pkg_s7.vhd" \
  "${model}/blk_clocks_${model}.vhd" \
  "common/blk_ds_simple_test.vhd" \
  "common/blk_clock_freq.vhd" \
  "${model}/blk_ds_clk_read_${model}.vhd" \
  "${model}/blk_ds_io_test_${model}.vhd" \
  "common/blk_ds_int_test.vhd" \
  "common/blk_ds_mem_reg.vhd" \
  "common/blk_ds_info.vhd" \
  "common/blk_direct_slave.vhd" \
  "common/bram_single_wrap_s7.vhd" \
  "common/blk_bram.vhd" \
  "common/blk_dma_switch.vhd" \
  "common/blk_mem_app.vhd" \
  "${model}/blk_mem_if_${model}.vhd" \
  "common/uber_s7.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_int_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_dly_cnt.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_timing_chk.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_main_fsm.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_init_fsm.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram.vhd" \
  "../../common/adb3_ocp/adb3_ocp_tb_pkg.vhd" \
  "../../common/adb3_probe/adb3_probe_pkg.vhd" \
  "../../common/adb3_probe/adb3_ocp_transaction_probe_sim.vhd" \
  "../../common/adb3_target/${model}/adb3_target_tb_inc_pkg_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_bridge/mptl_if_bridge_wrap_sim_${model}.vhd" \
  "../../common/adb3_target/${model}/test_board_clks_${model}.vhd" \
  "${model}/uber_tb_pkg_${model}.vhd" \
  "common/test_uber_ds.vhd" \
  "common/test_uber_dm.vhd" \
  "common/test_uber_dma_1ch_nb.vhd" \
  "common/test_uber_probes.vhd" \
  "${model}/test_uber_mem_${model}.vhd" \
  "common/test_uber.vhd" \
  "common/tb_wrap.v" \
]

# Simulation sources for OCP-only 'light weight' behavioural simulation
set src_sim_lwb [ list \
  "../../common/adb3_ocp/adb3_ocp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_comp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_cross_clk_dom.vhd" \
  "../../common/adb3_ocp/adb3_ocp_retime_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_b.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_mux_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_ocp2ddr3_nb_s7.vhd" \
  "../../common/adb3_target/${model}/adb3_target_inc_pkg_sim_ocp_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_target/mptl_if_target_wrap_sim_${model}.vhd" \
  "../../common/ddr3_sdram_if/ddr3_if_pkg_v7.vhd" \
  "../../common/ddr3_sdram_if/lwb/mig_ddr3_lwb_s7.vhd" \
  "../../common/ddr3_sdram_if/lwb/ddr3_if_bank_lwb_s7.vhd" \
  "../../common/mem_apps/blk_mem_test.vhd" \
  "../../common/chipscope/blk_chipscope_sim.vhd" \
  "planahead/today_pkg_${model}_${device}.vhd" \
  "common/uber_pkg_s7.vhd" \
  "${model}/blk_clocks_${model}.vhd" \
  "common/blk_ds_simple_test.vhd" \
  "common/blk_clock_freq.vhd" \
  "${model}/blk_ds_clk_read_${model}.vhd" \
  "${model}/blk_ds_io_test_${model}.vhd" \
  "common/blk_ds_int_test.vhd" \
  "common/blk_ds_mem_reg.vhd" \
  "common/blk_ds_info.vhd" \
  "common/blk_direct_slave.vhd" \
  "common/bram_single_wrap_s7.vhd" \
  "common/blk_bram.vhd" \
  "common/blk_dma_switch.vhd" \
  "common/blk_mem_app.vhd" \
  "${model}/blk_mem_if_${model}.vhd" \
  "common/uber_s7.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_int_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_lwb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_tb_pkg.vhd" \
  "../../common/adb3_probe/adb3_probe_pkg.vhd" \
  "../../common/adb3_probe/adb3_ocp_transaction_probe_sim.vhd" \
  "../../common/adb3_target/${model}/adb3_target_tb_inc_pkg_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_bridge/mptl_if_bridge_wrap_sim_${model}.vhd" \
  "../../common/adb3_target/${model}/test_board_clks_${model}.vhd" \
  "${model}/uber_tb_pkg_${model}.vhd" \
  "common/test_uber_ds.vhd" \
  "common/test_uber_dm.vhd" \
  "common/test_uber_dma_1ch_nb.vhd" \
  "common/test_uber_probes.vhd" \
  "${model}/test_uber_mem_${model}.vhd" \
  "common/test_uber.vhd" \
]

# Simulation sources for full MPTL simulation
set src_sim_mptl ${src_mig_ddr3}
lappend src_sim_mptl [ list \
  "../../common/adb3_ocp/adb3_ocp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_comp_pkg.vhd" \
  "../../common/adb3_ocp/adb3_ocp_cross_clk_dom.vhd" \
  "../../common/adb3_ocp/adb3_ocp_retime_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if.vhd" \
  "../../common/adb3_ocp/adb3_ocp_simple_bus_if_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_b.vhd" \
  "../../common/adb3_ocp/adb3_ocp_split_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_mux_nb.vhd" \
  "../../common/adb3_ocp/adb3_ocp_ocp2ddr3_nb_s7.vhd" \
  "../../common/adb3_target/${model}/adb3_target_inc_pkg_syn_ngc_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_target/mptl_if_target_netlist_${model}${gt_type}.vhd" \
  "../../common/adb3_target/${model}/mptl_target/mptl_if_target_wrap_${model}${gt_type}.vhd" \
  "../../common/ddr3_sdram_if/ddr3_if_pkg_v7.vhd" \
  "../../common/ddr3_sdram_if/${model}/mig7_v${mig_ver}/ddr3_if_bank_${model}_mig7_v${mig_ver}.vhd" \
  "../../common/mem_apps/blk_mem_test.vhd" \
  "../../common/chipscope/blk_chipscope_sim.vhd" \
  "planahead/today_pkg_${model}_${device}.vhd" \
  "common/uber_pkg_s7.vhd" \
  "${model}/blk_clocks_${model}.vhd" \
  "common/blk_ds_simple_test.vhd" \
  "common/blk_clock_freq.vhd" \
  "${model}/blk_ds_clk_read_${model}.vhd" \
  "${model}/blk_ds_io_test_${model}.vhd" \
  "common/blk_ds_int_test.vhd" \
  "common/blk_ds_mem_reg.vhd" \
  "common/blk_ds_info.vhd" \
  "common/blk_direct_slave.vhd" \
  "common/bram_single_wrap_s7.vhd" \
  "common/blk_bram.vhd" \
  "common/blk_dma_switch.vhd" \
  "common/blk_mem_app.vhd" \
  "${model}/blk_mem_if_${model}.vhd" \
  "common/uber_s7.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_int_pkg.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_dly_cnt.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_timing_chk.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_main_fsm.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram_init_fsm.vhd" \
  "../../common/mem_tb/ddr3_sdram/ddr3_sdram.vhd" \
  "../../common/adb3_ocp/adb3_ocp_tb_pkg.vhd" \
  "../../common/adb3_probe/adb3_probe_pkg.vhd" \
  "../../common/adb3_probe/adb3_ocp_transaction_probe_sim.vhd" \
  "../../common/adb3_target/${model}/adb3_target_tb_inc_pkg_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_bridge/mptl128_gtx6_interface.vhd" \
  "../../common/adb3_target/${model}/mptl_bridge/mptl_if_bridge_netlist_${model}.vhd" \
  "../../common/adb3_target/${model}/mptl_bridge/mptl_if_bridge_wrap_${model}.vhd" \
  "../../common/adb3_target/${model}/test_board_clks_${model}.vhd" \
  "${model}/uber_tb_pkg_${model}.vhd" \
  "common/test_uber_ds.vhd" \
  "common/test_uber_dm.vhd" \
  "common/test_uber_dma_1ch_nb.vhd" \
  "common/test_uber_probes.vhd" \
  "${model}/test_uber_mem_${model}.vhd" \
  "common/test_uber.vhd" \
  "common/tb_wrap.v" \
]

# Work around an ISIM issue concerning mixed VHDL/Verilog simulation
#set vlog_defs_sim_ocp TB_WRAP_ENTITY=test_uber
#set vlog_defs_sim_mptl TB_WRAP_ENTITY=test_uber
set top_sim_ocp tb_wrap
set top_sim_mptl tb_wrap
